// 
// Copyright (c) 2004-2006 Jaroslaw Kowalski <jaak@jkowalski.net>
// 
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions 
// are met:
// 
// * Redistributions of source code must retain the above copyright notice, 
//   this list of conditions and the following disclaimer. 
// 
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution. 
// 
// * Neither the name of Jaroslaw Kowalski nor the names of its 
//   contributors may be used to endorse or promote products derived from this
//   software without specific prior written permission. 
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
// THE POSSIBILITY OF SUCH DAMAGE.
// 

using System;
using System.Text;
using System.Collections;
using System.Collections.Specialized;
using System.Reflection;
using System.Globalization;

using NLog.Internal;
using NLog.Layouts;
using NLog.Config;

namespace NLog
{
    /// <summary>
    /// A factory for layouts. Creates new layout instances based on their names.
    /// </summary>
    public sealed class LayoutFactory
    {
        private static TypeDictionary _name2type = new TypeDictionary();

        static LayoutFactory()
        {
            foreach (Assembly a in ExtensionUtils.GetExtensionAssemblies())
            {
                AddLayoutsFromAssembly(a, "");
            }
        }

        private LayoutFactory(){}

        /// <summary>
        /// Removes all layout information from the factory.
        /// </summary>
        public static void Clear()
        {
            _name2type.Clear();
        }

        /// <summary>
        /// Scans the specified assembly for types marked with <see cref="LayoutAttribute" /> and adds
        /// them to the factory. Optionally it prepends the specified text to layout names to avoid
        /// naming collisions.
        /// </summary>
        /// <param name="theAssembly">The assembly to be scanned for layouts.</param>
        /// <param name="prefix">The prefix to be prepended to layout names.</param>
        public static void AddLayoutsFromAssembly(Assembly theAssembly, string prefix)
        {
            try
            {
                InternalLogger.Debug("AddLayoutsFromAssembly('{0}')", theAssembly.FullName);
                foreach (Type t in theAssembly.GetTypes())
                {
                    LayoutAttribute[]attributes = (LayoutAttribute[])t.GetCustomAttributes(typeof(LayoutAttribute), false);
                    if (attributes != null)
                    {
                        foreach (LayoutAttribute attr in attributes)
                        {
                            if (PlatformDetector.IsSupportedOnCurrentRuntime(t))
                            {
                                AddLayout(prefix + attr.Name, t);
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                InternalLogger.Error("Failed to add layouts from '" + theAssembly.FullName + "': {0}", ex);
            }
        }

        /// <summary>
        /// Registers the specified layout type to the factory under a specified name.
        /// </summary>
        /// <param name="name">The name of the layout (e.g. <code>XML</code>, <code>CSV</code> or <code>HTML</code>)</param>
        /// <param name="t">The type of the new layout</param>
        /// <remarks>
        /// The name specified in the name parameter can then be used
        /// to reference layouts.
        /// </remarks>
        public static void AddLayout(string name, Type t)
        {
            InternalLogger.Trace("Registering layout {0} for type '{1}'", name, t.FullName);
            _name2type[name.ToLower()] = t;
        }

        /// <summary>
        /// Creates the layout renderer object based on its layout renderer name and sets its properties from parameters string.
        /// </summary>
        /// <param name="name">The name of the layout renderer (e.g. <code>message</code> or <code>aspnet-request</code>)</param>
        /// <returns>A new instance of the <see cref="ILayout"/> object.</returns>
        public static ILayout CreateLayout(string name)
        {
            Type t = _name2type[name.ToLower()];
            if (t != null)
            {
                ILayout la = FactoryHelper.CreateInstance(t) as ILayout;
                if (la != null)
                    return la;
            }
            throw new ArgumentException("Layout " + name + " not found.");
        }

        /// <summary>
        /// Collection of layout types added to the factory.
        /// </summary>
        public static ICollection LayoutTypes
        {
            get { return _name2type.Values; }
        }
    }
}
